export default class Query {
  constructor(source) {
    this.PATH = ''
    this.LIMIT = 10
    this.SOURCE = source
    this.RESULT = []
    this.WHERE_KEY = ''
    this.WHERE_LIKE = ''
    this.WHERE_FIRST = ''
    this.WHERE_SECOND = ''
    this.SORT_KEY = ''
    this.SORT_VAL = ''
  }

  select(path) {
    this.PATH = path

    return this
  }

  sort(path, type) {
    this.SORT_KEY = path
    this.SORT_VAL = type

    return this
  }

  limit (limit) {
    this.LIMIT = limit

    return this
  }

  where(property, value) {
    this.WHERE_KEY = property
    this.WHERE_FIRST = value

    return this
  }

  findOneBy(property, value) {
    this.WHERE_KEY = property
    this.WHERE_FIRST = value

    return this
  }

  whereBetween(property, first, second) {
    this.WHERE_KEY = property
    this.WHERE_FIRST = first
    this.WHERE_SECOND = second

    return this
  }

  whereLike(property, like) {
    this.WHERE_KEY = property
    this.WHERE_LIKE = like

    return this
  }

  get() {
    let {
      PATH,
      LIMIT,
      SOURCE,
      RESULT,
      WHERE_KEY,
      WHERE_LIKE,
      WHERE_FIRST,
      WHERE_SECOND,
      SORT_KEY,
      SORT_VAL 
    } = this

    RESULT = SOURCE

    if (WHERE_KEY !== '' && (WHERE_FIRST !== '' || WHERE_SECOND !== '')) {
      RESULT = SOURCE.filter((item, index) => {
        if (WHERE_LIKE) {
          return item[WHERE_KEY].includes(WHERE_LIKE)
        }
        else if (!WHERE_SECOND) {
          return item[WHERE_KEY] == WHERE_FIRST
        }
        else {
          return item[WHERE_KEY] >= WHERE_FIRST && item[WHERE_KEY] <= WHERE_SECOND
        }
      })
    }

    if (PATH !== '' && PATH !== '*') {
      const filter = PATH.split(',').map(item => item.trim())

      RESULT = RESULT.map(item => {
        const bracket = {}
  
        Object.keys(item).map(property => {
          if (filter.includes(property)) {
            bracket[property] = item[property]
          }
        })
  
        return bracket
      })
    }

    if (LIMIT > 0) {
      RESULT = RESULT.filter((item, index) => index < LIMIT)
    }

    if (SORT_KEY && ['asc', 'desc'].includes(SORT_VAL)) {
      const sortType =(first, second) => {
        if (first[SORT_KEY] < second[SORT_KEY])
          return SORT_VAL == 'desc' ? 1 : -1
        if (first[SORT_KEY] > second[SORT_KEY])
          return SORT_VAL == 'desc' ? -1 : 1
        return 0
      }
  
      const bracket = RESULT.sort(sortType)
  
      RESULT = bracket
    }

    return RESULT
  }
}
